# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import os, pickle, sysutils, syspaths

class AccountData:
    """
    Representa la información de una cuenta de e-mail.
    """

    def __init__(self, address, inboxSystem,
    imapServer, imapSsl, imapPort, imapUser, imapPassword,
    popServer, popSsl, popPort, popUser, popPassword,
    smtpServer, smtpTls, smtpPort, smtpUser, smtpPassword,
    partSize, partWaitTime,
    inboxDir):
        """
        Constructor de la clase.

        `address` es la dirección de e-mail.

        `inboxSystem` indica el protocolo para recibir archivos a usar. Puede tomar
        2 valores: 'imap' (indica que debe usarse IMAP) o 'pop' (indica que debe usarse 'POP').

        `imapServer` es la dirección del servidor de IMAP.

        `imapSsl` es una variable lógica que indica si debe usarse una conexión segura mediante SSL
        con el servidor de IMAP.

        `imapPort` es el puerto con el que conectar con el servidor de IMAP.

        `imapUser` es el nombre de usuario con el que conectar con el servidor de IMAP.

        `imapPassword` es la contraseña del usuario `imapUser` del servidor de IMAP.

        `popServer` es la dirección del servidor de POP.

        `popSsl` es una variable lógica que indica si debe usarse una conexión segura mediante SSL
        con el servidor de POP.

        `popPort` es el puerto con el que conectar con el servidor de POP.

        `popUser` es el nombre de usuario con el que conectar con el servidor de POP.

        `popPassword` es la contraseña del usuario `popUser` del servidor de POP.

        `smtpServer` es la dirección del servidor de SMTP.

        `smtpTls` es una variable lógica que indica si debe usarse una conexión segura mediante TLS
        con el servidor de SMTP.

        `smtpPort` es el puerto con el que conectar con el servidor de SMTP.

        `smtpUser` es el nombre de usuario con el que conectar con el servidor de SMTP.

        `smtpPassword` es la contraseña del usuario `smtpUser` del servidor de SMTP.

        `partSize` es el tamaño en kilobytes que debe tener cada una de las partes de un archivo
        al dividirlo para ser enviado.

        `partWaitTime` es el tiempo en segundos que hay que esperar entre el envío de una parte y otra.

        `inboxDir` es la ruta del directorio de bandeja de entrada. En este directorio es donde se almacenarán
        los archivos descargados completamente.
        """

        self.address = address
        self.inboxSystem = inboxSystem

        self.imapServer = imapServer
        self.imapSsl = imapSsl
        self.imapPort = imapPort
        self.imapUser = imapUser
        self.imapPassword = imapPassword

        self.popServer = popServer
        self.popSsl = popSsl
        self.popPort = popPort
        self.popUser = popUser
        self.popPassword = popPassword

        self.smtpServer = smtpServer
        self.smtpTls = smtpTls
        self.smtpPort = smtpPort
        self.smtpUser = smtpUser
        self.smtpPassword = smtpPassword
        
        self.partSize = partSize
        self.partWaitTime = partWaitTime

        self.inboxDir = inboxDir

    def check(self):
        """
        Comprueba si se tienen los datos necesarios. Devuelve `True` si se tienen datos necesarios
        y `False` en caso contrario.
        """

        cond1 = self.address != ""
        cond2 = self.inboxSystem != ""

        cond3 = self.imapServer != ""
        cond4 = self.imapUser != ""
        cond5 = self.imapPassword != ""

        cond6 = self.popServer != ""
        cond7 = self.popUser != ""
        cond8 = self.popPassword != ""

        cond9 = self.smtpServer != ""
        cond10 = self.smtpUser != ""
        cond11 = self.smtpPassword != ""
        
        cond12 = self.inboxDir != ""

        success = cond1 and cond2 and cond3 and cond4 and cond5 and cond6 and cond7 and cond8 and cond9 and cond10 and cond11 and cond12
        
        return success

class ConfigurationManager:
    """
    Contiene la lógica para el manejo de la configuración del usuario.
    """

    def __init__(self):
        """
        Constructor de la clase.
        """

        self.accountDataPath = syspaths.get_accountdata_file()

    def save_account_data(self, data):
        """
        Guarda los datos de la cuenta de e-mail del usuario.
        Los datos están almacenados en un objeto de tipo `AccountData`.

        `data` es el objeto de tipo `AccountData` a guardar. Se guarda serializándolo.
        """
        
        try:
            fileOutput = open(self.accountDataPath, 'wb')
            pickle.dump(data, fileOutput)
            fileOutput.close()
        except:
            raise Exception("Error saving account data.")

    def load_account_data(self):
        """
        Carga los datos de la cuenta de e-mail del usuario.
        Devuelve un objeto de tipo `AccountData`.

        El objeto que devuelve se obtiene deserializándolo.
        """

        try:
            fileInput = open(self.accountDataPath, 'rb')
            data = pickle.load(fileInput)
            fileInput.close()
        except:
            raise Exception("Error loading account data.")

        return data

    def get_default_account_data(self):
        """
        Devuelve los datos por defecto de la cuenta de e-mail.

        Devuelve un objeto de tipo `AccountData` con algunos
        parámetros establecidos.
        """

        return AccountData("", "imap",
        "", True, 993, "", "",
        "", True, 995, "", "",
        "", True, 587, "", "",
        1024*1024*15, 0, syspaths.get_default_inbox_dir())
