#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, pygtk, gtk, gtk.glade, sysutils, configuration, fileslogic, imap, pop, smtp, imaplogic, poplogic
import smtplogic, syspaths

class DialAccount:
    """
    Clase que representa la ventana de configuración de la cuenta de e-mail de la interfaz de usuario gráfica.
    """

    def __init__(self, accountData):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "ui/dialaccount.glade")
        self.builder = gtk.Builder()
        self.builder.add_from_file(self.gladefile)
        self.window = self.builder.get_object("dialAccount")
        self.entAddress = self.builder.get_object("entAddress")
        self.radImap = self.builder.get_object("radImap")
        self.radPop = self.builder.get_object("radPop")
        self.entImapServer = self.builder.get_object("entImapServer")
        self.chkImapSsl = self.builder.get_object("chkImapSsl")
        self.entImapPort = self.builder.get_object("entImapPort")
        self.entImapUser = self.builder.get_object("entImapUser")
        self.entImapPassword = self.builder.get_object("entImapPassword")
        self.entPopServer = self.builder.get_object("entPopServer")
        self.chkPopSsl = self.builder.get_object("chkPopSsl")
        self.entPopPort = self.builder.get_object("entPopPort")
        self.entPopUser = self.builder.get_object("entPopUser")
        self.entPopPassword = self.builder.get_object("entPopPassword")
        self.spinPartsSize = self.builder.get_object("spinPartsSize")
        self.spinPartsWaitTime = self.builder.get_object("spinPartsWaitTime")
        self.entInboxFolderPath = self.builder.get_object("entInboxFolderPath")
        self.entSmtpServer = self.builder.get_object("entSmtpServer")
        self.chkSmtpTls = self.builder.get_object("chkSmtpTls")
        self.entSmtpPort = self.builder.get_object("entSmtpPort")
        self.entSmtpUser = self.builder.get_object("entSmtpUser")
        self.entSmtpPassword = self.builder.get_object("entSmtpPassword")        
        self.builder.connect_signals(self)

        self.entAddress.set_text(accountData.address)
        if (accountData.inboxSystem == "imap"):
            self.radImap.set_active(True)
        else:
            self.radImap.set_active(False)
        self.entImapServer.set_text(accountData.imapServer)
        self.chkImapSsl.set_active(accountData.imapSsl)
        self.entImapPort.set_text(str(accountData.imapPort))
        self.entImapUser.set_text(accountData.imapUser)
        self.entImapPassword.set_text(accountData.imapPassword)
        self.entImapPassword.set_visibility(False)
        self.entPopServer.set_text(accountData.popServer)
        self.chkPopSsl.set_active(accountData.popSsl)
        self.entPopPort.set_text(str(accountData.popPort))
        self.entPopUser.set_text(accountData.popUser)
        self.entPopPassword.set_text(accountData.popPassword)
        self.entPopPassword.set_visibility(False)
        self.spinPartsSize.set_value(accountData.partSize / (1024 * 1024))
        self.spinPartsWaitTime.set_value(accountData.partWaitTime)
        self.entInboxFolderPath.set_text(accountData.inboxDir)
        self.entSmtpServer.set_text(accountData.smtpServer)
        self.chkSmtpTls.set_active(accountData.smtpTls)
        self.entSmtpPort.set_text(str(accountData.smtpPort))
        self.entSmtpUser.set_text(accountData.smtpUser)
        self.entSmtpPassword.set_text(accountData.smtpPassword)
        self.entSmtpPassword.set_visibility(False)

        self.window.show_all()        

        self.accountData = None

    def run(self):
        self.result = self.window.run()
        
        address = self.entAddress.get_text()
        
        if self.radImap.state == 1:
            inboxSystem = "imap"
        else:
            inboxSystem = "pop"

        imapServer = self.entImapServer.get_text()
        imapSsl = self.chkImapSsl.get_active()
        imapPort = int(self.entImapPort.get_text())
        imapUser = self.entImapUser.get_text()
        imapPassword = self.entImapPassword.get_text()

        popServer = self.entPopServer.get_text()
        popSsl = self.chkPopSsl.get_active()
        popPort = int(self.entPopPort.get_text())
        popUser = self.entPopUser.get_text()
        popPassword = self.entPopPassword.get_text()

        smtpServer = self.entSmtpServer.get_text()
        smtpTls = self.chkSmtpTls.get_active()
        smtpPort = int(self.entSmtpPort.get_text())
        smtpUser = self.entSmtpUser.get_text()
        smtpPassword = self.entSmtpPassword.get_text()

        partSize = int(self.spinPartsSize.get_value() * (1024*1024))
        partWaitTime = int(self.spinPartsWaitTime.get_value())

        inboxDir = self.entInboxFolderPath.get_text()

        self.accountData = configuration.AccountData(address, inboxSystem,
        imapServer, imapSsl, imapPort, imapUser, imapPassword,
        popServer, popSsl, popPort, popUser, popPassword,
        smtpServer, smtpTls, smtpPort, smtpUser, smtpPassword,
        partSize, partWaitTime,
        inboxDir)
        
        self.window.destroy()
        return self.result, self.accountData

    def butinboxfolderpath_clicked(self, widget):
        d = gtk.FileChooserDialog(title="Choose the inbox folder",action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        response = d.run()
        if response == gtk.RESPONSE_OK:
            self.entInboxFolderPath.set_text(d.get_current_folder())
        d.destroy()
