# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, math

def split_file(fromFile, toDir, partSize):
    """
    Divide un archivo en varias partes. Crea un archivo nuevo por cada parte.

    `fromFile` es la ruta del archivo a dividir.

    `toDir` es la ruta del directorio donde guardar las partes.

    `partSize` es el tamaño de cada parte en kilobytes.
    """
    fromFileName = os.path.basename(fromFile)
    fromFileSize = os.path.getsize(fromFile)
    partsNeeded = int(math.ceil(fromFileSize / partSize))
    maxChars = len(str(partsNeeded))
    partNum = 0
    input = open(fromFile, "rb")
    while 1:
        part = input.read(partSize)
        if not part: break;
        partNum = partNum + 1
        n = str(partNum)
        while (len(n) < maxChars):
            n = "0" + n
        fileName =  fromFileName + ".part" + n
        filePath = os.path.join(toDir, fileName)
        file = open(filePath, "wb")
        file.write(part)
        file.close()
    input.close()

def join_files(fromDir, toFile):
    """
    Une todos los archivos de un directorio en un solo archivo.

    `fromDir` es la ruta del directorio donde se encuentran los archivos a unir.

    `toFile` es la ruta del archivo que se creará, con el contenido de todos los archivos de `fromDir`.
    """
    readSize = 1024
    output = open(toFile, "wb")
    parts = os.listdir(fromDir)
    parts.sort()
    for fileName in parts:
        filePath = os.path.join(fromDir, fileName)
        if os.path.isfile(filePath):
            file = open(filePath, "rb")
            while 1:
                bytes = file.read(readSize)
                if not bytes: break;
                output.write(bytes)
            file.close()
    output.close()
