# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, files, sysutils, syspaths

class FilesLogic:
    """
    Clase para el manejo de los archivos a enviar y los archivos descargados.
    """

    def __init__(self, accountData):
        """
        Constructor de la clase.

        `accountData` es la configuración de la cuenta del usuario. Se requiere
        para obtener el directorio para los archivos recibidos (el directorio de bandeja de entrada)
        y para obtener más adelante el tamaño de cada parte a la hora de dividir y enviar un archivo.
        """

        self.accountData = accountData
        self.tempOutputDir = syspaths.get_temp_output_dir()
        self.tempInputDir = syspaths.get_temp_input_dir()
        self.inboxDir = self.accountData.inboxDir

    def split_file(self, filePath):
        """
        Divide el archivo que se quiere enviar en varias partes, para, después,
        ir enviando por e-mail cada parte mediante un método de la clase `SmtpLogic`.

        `filePath` es la ruta del archivo que se quiere enviar y que será dividido.

        Las partes se almacenan temporalmente (hasta que se envien todas) en la carpeta
        temporal del programa y cada parte es del tamaño especificado en la configuración
        de la cuenta (`accountData`).
        """

        files.split_file(filePath, self.tempOutputDir, self.accountData.partSize)

    def join_temp_files(self):
        """
        Une las partes recibidas de un archivo (que están en la carpeta temporal)
        en un solo archivo, almacenándolo en la carpeta de bandeja de entrada.
        """

        filesList = os.listdir(self.tempInputDir)
        
        for element in filesList:
            filePath = os.path.join(self.tempInputDir, element)
            if not os.path.isfile(filePath):
                filesList.remove(element)
                
        anyFile = filesList[0]    # The name of any file in the temporal directory
        i = anyFile.rfind(".")
        destFilePath = os.path.join(self.inboxDir, anyFile[:i])
        files.join_files(self.tempInputDir, destFilePath)
        
    def remove_temp_output_files(self):
        """
        Borra todos los archivos de la carpeta temporal de archivos a enviar.
        """

        for fileName in os.listdir(self.tempOutputDir):
            filePath = os.path.join(self.tempOutputDir, fileName)
            if os.path.isfile(filePath):
                os.remove(filePath)
                
    def remove_temp_input_files(self):
        """
        Borra todos los archivos de la carpeta temporal de archivos descargados.
        """
        
        for fileName in os.listdir(self.tempInputDir):
            filePath = os.path.join(self.tempInputDir, fileName)
            if os.path.isfile(filePath):
                os.remove(filePath)
