# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, email, smtplib
from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.Utils import formatdate
from email import Encoders

class SmtpAccount:
    """
    Clase básica para enviar mensajes de e-mail por medio de un servidor de SMTP.
    """

    def __init__(self, server, tls, port, user, password):
        """
        Constructor de la clase.
        
        `server` es la dirección del servidor con el que conectar.

        `tls` es una variable de tipo lógico que especifica si se debe usar una conexión
        segura por medio de TLS (depende del servidor).

        `port` es el puerto a usar para realizar la conexión (depende del servidor).

        `user` es el nombre de usuario con el que conectarse al servidor.

        `password` es la contraseña del usuario `user`.
        """

        self.server = server
        self.tls = tls
        self.port = port
        self.user = user
        self.password = password

    def connect(self):
        """
        Realiza la conexión con el servidor.
        """

        self.connection = smtplib.SMTP(self.server, self.port)
        self.connection.ehlo()
        if self.tls:
            self.connection.starttls()
            self.connection.ehlo()
        self.connection.login(self.user, self.password)

    def close(self):
        """
        Cierra la conexión con el servidor.

        Para llamar a este método, debe haber una conexión abierta.
        """

        if self.connection != None:
            self.connection.close()
            self.connection = None

    def send_message(self, fromAddress, toAddress, subject, text, attachments = []):
        """
        Envía un mensaje de e-mail.
        
        `fromAddress` es la dirección de e-mail que envía el mensaje (dirección origen).

        `toAddress` es la dirección a la que se envía el mensaje (dirección destino).

        `subject` es el asunto del mensaje.

        `text` es el cuerpo del mensaje.

        `attachments` es un array con las rutas de los archivos adjuntos.

        Para llamar a este método, debe haber una conexión abierta.
        """

        message = MIMEMultipart()
        message["From"] = fromAddress
        message["To"] = toAddress
        message["Date"] = formatdate(localtime = True)
        message["Subject"] = subject
        message.attach(MIMEText(text))

        for a in attachments:
            part = MIMEBase("application", "octet-stream")
            f = open(a, "rb")
            content = f.read()
            f.close()
            part.set_payload(content)
            Encoders.encode_base64(part)
            part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(a))
            message.attach(part)

        self.connection.sendmail(fromAddress, toAddress, message.as_string())
