# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, time, excepts, sysutils, syspaths

class SmtpLogic:
    """
    Clase que encapsula la lógica a la hora de enviar un archivo.

    Utiliza la clase básica `SmtpAccount`.
    """

    tempDir = syspaths.get_temp_output_dir()

    def __init__(self, account, partWaitTime):
        """
        Constructor de la clase.

        `account` es un objeto de tipo `SmtpAccount`. Se utilizará para enviar
        cada una de las partes del archivo total a enviar.

        `partWaitTime` es el tiempo de espera en segundos entre el envío de una parte y otra.
        Debe ser, al menos, 0.
        """

        self.account = account
        self.partWaitTime = partWaitTime

    def send_file(self, filePath, fromAddress, toAddress):
        """
        Envía un archivo, enviando cada una de sus partes.

        `filePath` es la ruta del archivo total a enviar.

        `fromAddress` es la dirección de e-mail desde la que se envían las partes (dirección origen).

        `toAddress` es la dirección de e-mail a la que se envían las partes (dirección destino).
        """

        yield 0
        partFiles = os.listdir(self.tempDir)
        for e in partFiles:
            if os.path.isdir(e):
                partFiles.remove(e)
        partFiles.sort()
        partCount = len(partFiles)
        i = partFiles[0].rfind(".")
        fileName = partFiles[0][:i]
        now = time.time()
        
        subject = "[unimail-msg][t]" + str(now) + "[t][p]" + str(partCount) + "[p]" + fileName
        path = os.path.join(self.tempDir, partFiles[0])
        part = 0
        
        for i in range(0, partCount):
            path = os.path.join(self.tempDir, partFiles[i])
            error = True
            tr = 0
            while error and tr < 6:            
                try:
                    self.account.send_message(fromAddress, toAddress, subject, "", [path])
                    error = False
                    time.sleep(self.partWaitTime)
                except:
                    tr += 1
                    if tr == 6:
                        raise excepts.ConnectionException("Error downloading file")
                    self.account.connect()
                    error = True
                    
            part += 1
            yield ((100*part)/partCount)
