#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, pwd, gobject
sys.dont_write_bytecode = True
from daemon import Daemon
import dbus
import pop, imap, fileslogic, poplogic, imaplogic, configuration

class ReceiveFileDaemon(Daemon):
    def __init__(self, fileIndex=None, dbusObj=None):
        currentUser = pwd.getpwuid(os.getuid())[0]
        pidFile = "/tmp/unimail-receive-daemon-{0}.pid".format(currentUser)
        errFile = "/tmp/unimail-receive-daemon-{0}.err".format(currentUser)
        self.fileIndex = fileIndex
        self.dbusObj = dbusObj        
        Daemon.__init__(self, pidFile, stderr=errFile)

    def run(self):
        self.sessionBus = dbus.SessionBus()
        self.remoteObject = self.sessionBus.get_object("es.rediris.forja.unimail.receive", self.dbusObj)

        try:
            self.receive_file()
        except Exception as e:
            self.remoteObject.notificate(e.args[0])

    def receive_file(self):
        confManag = configuration.ConfigurationManager()
        
        try:
            accountData = confManag.load_account_data()
        except:
            raise Exception("configuration-load-error")
            
        filesLogic = fileslogic.FilesLogic(accountData)
        
        if accountData.inboxSystem == "imap":
            emailInboxAccount = imap.ImapAccount(accountData.imapServer, accountData.imapSsl,
                accountData.imapPort, accountData.imapUser, accountData.imapPassword)
            emailInboxLogic = imaplogic.ImapLogic(emailInboxAccount)
        else:
            emailInboxAccount = pop.PopAccount(accountData.popServer, accountData.popSsl,
                accountData.popPort, accountData.popUser, accountData.popPassword)
            emailInboxLogic = poplogic.PopLogic(emailInboxAccount)
            
        try:
            emailInboxAccount.connect()
        except:
            raise Exception("connection-error")
            
        self.remoteObject.notificate("connection-success")
        
        try:
            filesLogic.remove_temp_input_files()
        except:
            raise Exception("tempfiles-remove-error")
            
        try:
            for progress in emailInboxLogic.refresh():
                pass
            self.inboxFileList = emailInboxLogic.get_unimail_files_list()
        except:
            raise Exception("search-error")
        
        self.remoteObject.notificate("search-success")
                    
        try:
            for progress in emailInboxLogic.download_file(self.inboxFileList[self.fileIndex]["index"]):
                self.remoteObject.notificate("receive-progress", str(progress))
        except:
            raise Exception("receive-error")
            
        try:
            filesLogic.join_temp_files()
        except:
            raise Exception("file-join-error")
            
        try:
            filesLogic.remove_temp_input_files()
        except:
            raise Exception("tempfiles-remove-error")
            
        self.remoteObject.notificate("receive-success")
            
if __name__ == "__main__":
    if len(sys.argv) == 4:
        if sys.argv[1] == "start":
            fileIndex = int(sys.argv[2])
            dbusObj = sys.argv[3]
            daemon = ReceiveFileDaemon(fileIndex, dbusObj)
            daemon.start()
        elif sys.argv[1] == "restart":
            fileIndex = sys.argv[2]
            dbusObj = sys.argv[3]
            daemon = ReceiveFileDaemon(fileIndex, dbusObj)
            daemon.start()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    elif len(sys.argv) == 2:
        if sys.argv[1] == "stop":
            daemon = ReceiveFileDaemon()
            daemon.stop()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    else:
        print "Usage: %s start [file index] [D-BUS path]|stop|restart [file index] [D-BUS path]" % sys.argv[0]
        sys.exit(2)
