#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, pwd, gobject
sys.dont_write_bytecode = True
from daemon import Daemon
import dbus
import pop, imap, fileslogic, poplogic, imaplogic, configuration  
        
class SearchFilesDaemon(Daemon):
    def __init__(self, dbusObj=None):
        currentUser = pwd.getpwuid(os.getuid())[0]
        pidFile = "/tmp/unimail-search-daemon-{0}.pid".format(currentUser)
        errFile = "/tmp/unimail-search-daemon-{0}.err".format(currentUser)
        self.dbusObj = dbusObj
        Daemon.__init__(self, pidFile, stderr=errFile)
        
    def run(self):
        self.sessionBus = dbus.SessionBus()
        self.remoteObject = self.sessionBus.get_object("es.rediris.forja.unimail.search", self.dbusObj)

        try:
            fileList = self.search_inbox_files()
            self.remoteObject.set_inbox_list(fileList)
        except Exception as e:
            self.remoteObject.notificate(e.args[0]) 
            
    def search_inbox_files(self):     
        confManag = configuration.ConfigurationManager()
        
        try:
            accountData = confManag.load_account_data()
        except:
            raise Exception("configuration-load-error")
            
        filesLogic = fileslogic.FilesLogic(accountData)
        
        if accountData.inboxSystem == "imap":
            emailInboxAccount = imap.ImapAccount(accountData.imapServer, accountData.imapSsl,
                accountData.imapPort, accountData.imapUser, accountData.imapPassword)
            emailInboxLogic = imaplogic.ImapLogic(emailInboxAccount)
        else:
            emailInboxAccount = pop.PopAccount(accountData.popServer, accountData.popSsl,
                accountData.popPort, accountData.popUser, accountData.popPassword)
            emailInboxLogic = poplogic.PopLogic(emailInboxAccount)
            
        try:
            emailInboxAccount.connect()
        except:
            raise Exception("connection-error")
        
        self.remoteObject.notificate("connection-success")
        
        try:
            for progress in emailInboxLogic.refresh():
                self.remoteObject.notificate("search-progress", str(progress))
        except:
            raise Exception("search-error")
        
        try:
            fileList = emailInboxLogic.get_unimail_files_list()

            for f in fileList:
                f["index"] = str(f["index"])
                f["parts"] = str(f["parts"])
        except:
            raise Exception("list-get-error")
            
        return fileList
        
if __name__ == "__main__":
    if len(sys.argv) == 3:
        if sys.argv[1] == "start":
            dbusObj = sys.argv[2]
            daemon = SearchFilesDaemon(dbusObj)
            daemon.start()
        elif sys.argv[1] == "restart":
            dbusObj = sys.argv[2]
            daemon = SearchFilesDaemon(dbusObj)
            daemon.restart()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    elif len(sys.argv) == 2:
        if sys.argv[1] == "stop":
            daemon = SearchFilesDaemon()
            daemon.stop()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    else:
        print "Usage: %s start [D-BUS path]|stop|restart [D-BUS path]" % sys.argv[0]
        sys.exit(2)
