#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, pwd, gobject
sys.dont_write_bytecode = True
from daemon import Daemon
import dbus
import smtp, fileslogic, smtplogic, configuration

class SendFileDaemon(Daemon):
    def __init__(self, filePath=None, emailFromAddress=None, emailToAddress=None, dbusObj=None):
        currentUser = pwd.getpwuid(os.getuid())[0]
        pidFile = "/tmp/unimail-send-daemon-{0}.pid".format(currentUser)
        errFile = "/tmp/unimail-send-daemon-{0}.err".format(currentUser)
        self.dbusObj = dbusObj
        self.filePath = filePath
        self.emailFromAddress = emailFromAddress
        self.emailToAddress = emailToAddress
        Daemon.__init__(self, pidFile, stderr=errFile)
        
    def run(self):
        self.sessionBus = dbus.SessionBus()
        self.remoteObject = self.sessionBus.get_object("es.rediris.forja.unimail.send", self.dbusObj)

        try:
            self.send_file()
        except Exception as e:
            self.remoteObject.notificate(e.args[0])

    def send_file(self):
        confManag = configuration.ConfigurationManager()
        
        try:
            accountData = confManag.load_account_data()
        except:
            raise Exception("configuration-load-error")
            
        emailOutAccount = smtp.SmtpAccount(accountData.smtpServer, accountData.smtpTls, accountData.smtpPort, accountData.smtpUser, accountData.smtpPassword)
        
        try:
            emailOutAccount.connect()
        except:
            raise Exception("connection-error")
            
        self.remoteObject.notificate("connection-success")
        emailOutLogic = smtplogic.SmtpLogic(emailOutAccount, accountData.partWaitTime)
        filesLogic = fileslogic.FilesLogic(accountData)
        
        try:
            filesLogic.remove_temp_output_files()
        except:
            raise Exception("tempfiles-remove-error")
        
        try:
            filesLogic.split_file(self.filePath)
        except:
            raise Exception("file-split-error")
            
        try:
            for progress in emailOutLogic.send_file(self.filePath, self.emailFromAddress, self.emailToAddress):
                self.remoteObject.notificate("send-progress", str(progress))
        except:
            raise Exception("send-error")

        emailOutAccount.close()
        
        try:
            filesLogic.remove_temp_output_files()
        except:
            raise Exception("tempfiles-remove-error")
        
        self.remoteObject.notificate("send-success")

if __name__ == "__main__":
    if len(sys.argv) == 6:
        if sys.argv[1] == "start":
            filePath = sys.argv[2]
            emailFromAddress = sys.argv[3]
            emailToAddress = sys.argv[4]
            dbusObj = sys.argv[5]
            daemon = SendFileDaemon(filePath, emailFromAddress, emailToAddress, dbusObj)
            daemon.start()
        elif sys.argv[1] == "restart":
            filePath = sys.argv[2]
            emailFromAddress = sys.argv[3]
            emailToAddress = sys.argv[4]
            dbusObj = sys.argv[5]
            daemon = SendFileDaemon(filePath, emailFromAddress, emailToAddress, dbusObj)
            daemon.restart()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    elif len(sys.argv) == 2:
        if sys.argv[1] == "stop":
            daemon = SendFileDaemon()
            daemon.stop()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    else:
        print "Usage: %s start [file path] [e-mail from address] [e-mail to address] [D-BUS path]|stop|restart [file path] [e-mail from address] [e-mail to address] [D-BUS path]" % sys.argv[0]
        sys.exit(2)
