#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, copy, threading, gobject, gtk, commands, dbus, dbus.service, dbus.mainloop.glib
import sysutils, syspaths, configuration
import fileslogic, imap, pop, smtp, imaplogic, poplogic, smtplogic
import dialaccount, dialabout

class SearchNotificationService(dbus.service.Object):
    def __init__(self, winMain, bus, objectPath):
        dbus.service.Object.__init__(self, bus, objectPath)
        self.winMain = winMain
        
    @dbus.service.method("es.rediris.forja.unimail.search")
    def set_inbox_list(self, fileList):
        for i in range(0, len(fileList)):
            self.add_filelist_item([i+1, fileList[i]["file"], fileList[i]["from"]])
        
        self.winMain.notbPanels.set_sensitive(True)
        self.winMain.mitemConfiguration.set_sensitive(True)
        self.winMain.progbProgress.set_fraction(0)
        self.winMain.labProgress.set_text("")
        self.winMain.butRefresh.set_sensitive(True)
        self.winMain.progbProgress.hide()
        self.winMain.butStopOperation.hide()
                
        self.mode = None
        
    @dbus.service.method("es.rediris.forja.unimail.search")
    def notificate(self, messageType, message=None):        
        if messageType == "configuration-load-error":
            self.notificate_error("Configuration load error")
            self.reset_ui()
            self.mode = None
        elif messageType == "connection-error":
            self.notificate_error("Connection error")
            self.reset_ui()
            self.mode = None
        elif messageType == "connection-success":
            self.winMain.labProgress.set_text("Searching...")
        elif messageType == "search-progress":
            self.winMain.progbProgress.set_fraction(float(message)/100)
        elif messageType == "search-error":
            self.notificate_error("Search error")
            self.reset_ui()
            self.mode = None
        elif messageType == "list-get-error":
            self.notificate_error("Search error")
            self.reset_ui()
            self.mode = None
        elif messageType == "list-get-error":
            self.notificate_error("Getting file list error")
            self.reset_ui()          
            self.mode = None
        elif messageType == "error":
            self.notificate_error("Unknown error")
            self.reset_ui()
            self.mode = None
            
    def add_filelist_item(self, item):
        self.winMain.trvInboxModel.append(item)
            
    def reset_ui(self):
        self.winMain.notbPanels.set_sensitive(True)
        self.winMain.mitemConfiguration.set_sensitive(True)
        self.winMain.progbProgress.set_fraction(0)
        self.winMain.labProgress.set_text("")
        self.winMain.butRefresh.set_sensitive(True)
        self.winMain.progbProgress.hide()
        self.winMain.butStopOperation.hide()
        
    def notificate_error(self, text):
        dialError = gtk.MessageDialog(self.winMain.window, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, text)
        dialError.set_title("Error")
        dialError.run()
        dialError.destroy()
        
class SendNotificationService(dbus.service.Object):
    def __init__(self, winMain, bus, objectPath):
        dbus.service.Object.__init__(self, bus, objectPath)
        self.winMain = winMain
        
    @dbus.service.method("es.rediris.forja.unimail.send")
    def notificate(self, messageType, message=None):
        if messageType == "configuration-load-error":
            self.notificate_error("Configuration load error")
            self.reset_ui()          
            self.mode = None
        elif messageType == "connection-error":
            self.notificate_error("Connection error")
            self.reset_ui()          
            self.mode = None
        elif messageType == "connection-success":
            self.winMain.labProgress.set_text("Sending...")
        elif messageType == "send-progress":
            self.winMain.progbProgress.set_fraction(float(message)/100)
        elif messageType == "tempfiles-remove-error":
            self.notificate_error("Removing temporal files error")
            self.reset_ui()          
            self.mode = None
        elif messageType == "file-split-error":
            self.notificate_error("File splitting error")
            self.reset_ui()          
            self.mode = None
        elif messageType == "send-error":
            self.notificate_error("Sending file error")
            self.reset_ui()          
            self.mode = None
        elif messageType == "send-success":
            self.notificate_send_success()
            self.reset_ui()          
            self.mode = None
        elif messageType == "error":
            self.notificate_error("Unknown error")
            self.reset_ui()
            self.mode = None
            
    def reset_ui(self):
        self.winMain.notbPanels.set_sensitive(True)
        self.winMain.mitemConfiguration.set_sensitive(True)
        self.winMain.progbProgress.set_fraction(0)
        self.winMain.labProgress.set_text("")
        self.winMain.butRefresh.set_sensitive(True)
        self.winMain.progbProgress.hide()
        self.winMain.butStopOperation.hide()
        self.winMain.entFile.set_text("")
        self.winMain.entSendTo.set_text("")
        
    def notificate_send_success(self):
        dialError = gtk.MessageDialog(self.winMain.window, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "The file has been sent successfully")
        dialError.set_title("Send file")
        dialError.run()
        dialError.destroy()
        
    def notificate_error(self, text):
        dialError = gtk.MessageDialog(self.winMain.window, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, text)
        dialError.set_title("Error")
        dialError.run()
        dialError.destroy()
        
class ReceiveNotificationService(dbus.service.Object):
    def __init__(self, winMain, bus, objectPath):
        dbus.service.Object.__init__(self, bus, objectPath)
        self.winMain = winMain
        
    @dbus.service.method("es.rediris.forja.unimail.receive")
    def notificate(self, messageType, message=None):    
        if messageType == "configuration-load-error":
            self.notificate_error("Configuration load error")
            self.reset_ui()
            self.mode = None
        elif messageType == "connection-error":
            self.notificate_error("Connection error")
            self.reset_ui()
            self.mode = None
        elif messageType == "connection-success":
            self.winMain.labProgress.set_text("Searching...")
        elif messageType == "search-error":
            self.notificate_error("Searching error")
            self.reset_ui()
            self.mode = None
        elif messageType == "search-success":    
            self.winMain.labProgress.set_text("Receiving...")
        elif messageType == "receive-progress":
            self.winMain.progbProgress.set_fraction(float(message)/100)
        elif messageType == "tempfiles-remove-error":
            self.notificate_error("Removing temporal files error")
            self.reset_ui()
            self.mode = None
        elif messageType == "file-join-error":
            self.notificate_error("Joining temporal files error")
            self.reset_ui()
            self.mode = None
        elif messageType == "receive-error":
            self.notificate_error("Receiving file error")
            self.reset_ui()
            self.mode = None
        elif messageType == "receive-success":
            self.notificate_receive_success()
            self.reset_ui()
            self.mode = None
        elif messageType == "error":
            self.notificate_error("Unknown error")
            self.reset_ui()
            self.mode = None
            
    def reset_ui(self):
        path, focus_column = self.winMain.trvInbox.get_cursor()
        (self.winMain.trvInbox.get_selection()).unselect_path(path)
        self.winMain.notbPanels.set_sensitive(True)
        self.winMain.mitemConfiguration.set_sensitive(True)
        self.winMain.progbProgress.set_fraction(0)
        self.winMain.labProgress.set_text("")
        self.winMain.butRefresh.set_sensitive(True)
        self.winMain.progbProgress.hide()
        self.winMain.butStopOperation.hide()
            
    def notificate_receive_success(self):
        dialError = gtk.MessageDialog(self.winMain.window, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "The file has been received successfully")
        dialError.set_title("Receive file")
        dialError.run()
        dialError.destroy()
              
    def notificate_error(self, text):
        dialError = gtk.MessageDialog(self.winMain.window, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, text)
        dialError.set_title("Error")
        dialError.run()
        dialError.destroy()
        
class WinMain:
    """
    Clase que representa la ventana principal de la interfaz de usuario gráfica.
    """

    def __init__(self):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "ui/winmain.glade")
        self.builder = gtk.Builder()
        self.builder.add_from_file(self.gladefile)
        self.window = self.builder.get_object("winMain")
        self.labSessionState = self.builder.get_object("labSessionState")
        self.trvInbox = self.builder.get_object("trvInbox")
        self.progbProgress = self.builder.get_object("progbProgress")
        self.butStopOperation = self.builder.get_object("butStopOperation");
        self.labProgress = self.builder.get_object("labProgress")
        self.statbMain = self.builder.get_object("statbMain")
        self.notbPanels = self.builder.get_object("notbPanels")
        self.butRefresh = self.builder.get_object("butRefresh")
        self.butDownload = self.builder.get_object("butDownload")
        self.mitemConfiguration = self.builder.get_object("mitemConfiguration")
        self.entFile = self.builder.get_object("entFile")
        self.entSendTo = self.builder.get_object("entSendTo")
        self.butSend = self.builder.get_object("butSend")
        self.mitemConfiguration = self.builder.get_object("mitemConfiguration")
        self.menuTray = self.builder.get_object("menuTray")
        self.mitemOpen = self.builder.get_object("mitemOpen")
        self.builder.connect_signals(self)
        
        column1 = gtk.TreeViewColumn("ID", gtk.CellRendererText(), text=0)
        column1.set_resizable(True)
        self.trvInbox.append_column(column1)
        column2 = gtk.TreeViewColumn("File", gtk.CellRendererText(), text=1)
        column2.set_resizable(True)
        self.trvInbox.append_column(column2)
        column3 = gtk.TreeViewColumn("From", gtk.CellRendererText(), text=2)
        column3.set_resizable(True)
        self.trvInbox.append_column(column3)        
        self.trvInboxModel = gtk.ListStore(int, str, str)
        self.trvInbox.set_model(self.trvInboxModel)
        self.window.show_all()
        
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        
        self.searchBus = dbus.SessionBus()
        self.searchBusName = dbus.service.BusName("es.rediris.forja.unimail.search", self.searchBus)
        self.searchExposedObject = SearchNotificationService(self, self.searchBus, "/es/rediris/forja/unimail/search")
         
        self.sendBus = dbus.SessionBus()
        self.sendBusName = dbus.service.BusName("es.rediris.forja.unimail.send", self.sendBus)
        self.sendExposedObject = SendNotificationService(self, self.sendBus, "/es/rediris/forja/unimail/send")
        
        self.receiveBus = dbus.SessionBus()
        self.receiveBusName = dbus.service.BusName("es.rediris.forja.unimail.receive", self.receiveBus)
        self.receiveExposedObject = ReceiveNotificationService(self, self.receiveBus, "/es/rediris/forja/unimail/receive") 
         
        confManag = configuration.ConfigurationManager()
        self.accountData = confManag.load_account_data()   
        self.inboxFileList = []
        self.update_interface()
        self.update_logic()
    
    def winmain_deleteevent(self, widget, event, data=None):
        self.hide_window()
        return True
        
    def winmain_destroy(self, widget):
        gtk.main_quit()
        
    def update_interface(self):
        if self.accountData.address == "":
            self.labSessionState.set_text("Set your account configuration first")
        else:
            self.labSessionState.set_text(self.accountData.address)

        cond1 = (self.accountData.address != "")
        cond2 = (self.accountData.inboxSystem == "imap" and self.accountData.imapServer != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popServer != "")
        cond3 = (self.accountData.inboxSystem == "imap" and self.accountData.imapPort != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popPort != "")
        cond4 = (self.accountData.inboxSystem == "imap" and self.accountData.imapUser != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popUser != "")
        cond5 = (self.accountData.inboxSystem == "imap" and self.accountData.imapPassword != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popPassword != "")

        if cond1 and cond2 and cond3 and cond4 and cond5:
            self.notbPanels.set_sensitive(True)
        else:
            self.notbPanels.set_sensitive(False)

    def update_logic(self):
        self.filesLogic = fileslogic.FilesLogic(self.accountData)
        
        if self.accountData.inboxSystem == "imap":
            self.emailInboxAccount = imap.ImapAccount(self.accountData.imapServer, self.accountData.imapSsl,
                self.accountData.imapPort, self.accountData.imapUser, self.accountData.imapPassword)
            self.emailInboxLogic = imaplogic.ImapLogic(self.emailInboxAccount)
        else:
            self.emailInboxAccount = pop.PopAccount(self.accountData.popServer, self.accountData.popSsl,
                self.accountData.popPort, self.accountData.popUser, self.accountData.popPassword)
            self.emailInboxLogic = poplogic.PopLogic(self.emailInboxAccount)

    def entfile_changed(self, widget):
        self.check_send_data()
    
    def entsendto_changed(self, widget):
        self.check_send_data()

    def check_send_data(self):
        cond1 = self.entFile.get_text() != ""
        cond2 = self.entSendTo.get_text() != ""
        self.butSend.set_sensitive(cond1 and cond2)

    def imitemclose_activate(self, widget):
        self.hide_window()
        
    def mitemaccount_activate(self, widget):
        dialAccount = dialaccount.DialAccount(self.accountData)
        result, data = dialAccount.run()
        if result == 0:
            self.accountData = data
            confManag = configuration.ConfigurationManager()
            confManag.save_account_data(self.accountData)
            self.update_interface()
            self.update_logic()
        
    def imitemabout_activate(self, widget):
        dialAbout = dialabout.DialAbout()
        dialAbout.run()
    
    def trvinbox_cursor_changed(self, widget):
        selection, iterator = (self.trvInbox.get_selection()).get_selected()
        cond = len(selection) > 0
        self.butDownload.set_sensitive(cond)

    def butrefresh_clicked(self, widget):
        self.refresh_inbox()
        
    def refresh_inbox(self):
        self.trvInboxModel.clear()
        self.notbPanels.set_sensitive(False)
        self.mitemConfiguration.set_sensitive(False)
        self.labProgress.set_text("Connecting...")
        self.butRefresh.set_sensitive(False)
        self.butDownload.set_sensitive(False)
        self.progbProgress.show()
        self.butStopOperation.show()
        
        self.mode = "search"
        commands.getoutput("python unimail-search-daemon.py start '/es/rediris/forja/unimail/search'")

    def butsend_clicked(self, widget):
        filePath = self.entFile.get_text()
        fromAddress = self.accountData.address
        toAddress = self.entSendTo.get_text()
        
        self.notbPanels.set_sensitive(False)
        self.mitemConfiguration.set_sensitive(False)
        self.labProgress.set_text("Connecting...")
        self.butRefresh.set_sensitive(False)
        self.butDownload.set_sensitive(False)
        self.progbProgress.show()
        self.butStopOperation.show()
        
        self.mode = "send"
        commands.getoutput("python unimail-send-daemon.py start '{0}' '{1}' '{2}' '/es/rediris/forja/unimail/send'".format(filePath, fromAddress, toAddress))

    def butdownload_clicked(self, widget):
        selection, iterator = (self.trvInbox.get_selection()).get_selected()
        idSelected = selection.get_value(iterator, 0)
        
        self.notbPanels.set_sensitive(False)
        self.mitemConfiguration.set_sensitive(False)
        self.labProgress.set_text("Connecting...")
        self.butRefresh.set_sensitive(False)
        self.butDownload.set_sensitive(False)
        self.progbProgress.show()
        self.butStopOperation.show()

        self.mode = "receive"
        commands.getoutput("python unimail-receive-daemon.py start {0} '/es/rediris/forja/unimail/receive'".format(idSelected-1))
        
    def butstopoperation_clicked(self, widget):
        if self.mode == "search":
            commands.getoutput("python unimail-search-daemon.py stop")
        elif self.mode == "send":
            commands.getoutput("python unimail-send-daemon.py stop")
        elif self.mode == "receive":
            commands.getoutput("python unimail-receive-daemon.py stop")
        self.mode = None
        
        self.trvInboxModel.clear()
        self.notbPanels.set_sensitive(True)
        self.mitemConfiguration.set_sensitive(True)
        self.progbProgress.set_fraction(0)
        self.labProgress.set_text("")
        self.butRefresh.set_sensitive(True)
        self.progbProgress.hide()
        self.butStopOperation.hide()
        
    def butbrowse_clicked(self, widget):
        d = gtk.FileChooserDialog(title="Select file to send", action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        response = d.run()
        if response == gtk.RESPONSE_OK:
            self.entFile.set_text(d.get_filename())
        d.destroy()
        
    def hide_window(self):
        self.mitemOpen.set_sensitive(True)
        self.window.hide()
        
    def show_window(self):
        if not self.window.get_property("visible"):
            self.mitemOpen.set_sensitive(False)
            self.window.show()
            
    def sticonapp_popupmenu(self, widget, button, time, data=None):
        self.menuTray.show_all()
        self.menuTray.popup(None, None, None, 3, time)
        
    def sticonapp_activate(self, widget):
        self.show_window()
    
    def mitemopen_activate(self, widget):
        self.show_window()
        
    def mitemexit_activate(self, widget):
        gtk.main_quit()
